unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls, zbar;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnStart: TButton;
    mem: TMemo;
    Panel1: TPanel;
    stBar: TStatusBar;
    procedure btnStartClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    Procedure ZBarDataVisu(Barcode: string; Typ: zbar_symbol_type_t);
    Procedure ZBarTerminate(Sender: TObject);
  end;

  TNewZDataEvent = procedure(Barcode: string; Typ: zbar_symbol_type_t) of object;
  TThreadZBar = class(TThread)
  private
    fLog: String;
    fBarcode: string;
    fTyp: zbar_symbol_type_t;
    FOnNewZBar: TNewZDataEvent;
    ZBarProc: zbar_processor_t;
    procedure SendBarcode;
    procedure Termi;
    procedure DoLogExt;
  public
    constructor Create;
    procedure Execute; override;
    property OnNewZBar: TNewZDataEvent read FOnNewZBar write FOnNewZBar;
  end;
Procedure ZBarDataHandler(image: zbar_image_t; userdata: Pointer); cdecl;

var
  Form1: TForm1;
  thZBarcode: TThreadZBar;
  thZBarTerminated: Boolean;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.btnStartClick(Sender: TObject);
begin
  mem.Clear;
  stBar.Panels[0].Text := 'ZBar Start ... ';
  thZBarcode := TThreadZBar.Create;
  thZBarcode.OnNewZBar := @ZBarDataVisu;
  thZBarcode.OnTerminate := @ZBarTerminate;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Form1 := Self;
  mem.Clear;
end;

procedure TForm1.ZBarDataVisu(Barcode: string; Typ: zbar_symbol_type_t);
Var s: String;
Begin
  s := zbar_get_symbol_name(Typ) + ' ' + Barcode;
  stBar.Panels[0].Text := 'Barcode Read: ' + s;
  mem.Append('Barcode Read: ' + s);
end;

procedure TForm1.ZBarTerminate(Sender: TObject);
Begin
  stBar.Panels[0].Text := 'ZBar End.';
  thZBarcode := Nil;
end;

////////////////////////////////////////////////////////////////////
// Thread für Barcode Reader
constructor TThreadZBar.Create;
begin
  thZBarTerminated := False;
  inherited Create(False);
  FreeOnTerminate := True;
end;

procedure TThreadZBar.SendBarcode;
begin
  if Assigned(FOnNewZBar) then
    FOnNewZBar(fBarcode, fTyp);
end;

procedure TThreadZBar.Termi;
begin
  thZBarTerminated := True;
end;

procedure TThreadZBar.DoLogExt;
Begin
  Form1.stBar.Panels[0].Text := fLog;
  Form1.mem.Append(fLog);
end;

Procedure ZBarDataHandler(image: zbar_image_t; userdata: Pointer); cdecl; // Callback von DLL
Var sym : zbar_symbol_t;
Begin
  sym := zbar_image_first_symbol(image);
  while Assigned(sym) Do
  Begin
    if zbar_symbol_get_count(sym) = 0 Then
    Begin
      thZBarcode.fTyp := zbar_symbol_get_type(sym);
      if (thZBarcode.fTyp <> ZBAR_PARTIAL) Then
      Begin
        thZBarcode.fBarcode := zbar_symbol_get_data(sym);
        thZBarcode.Synchronize(@thZBarcode.SendBarcode);
      end;
    end;
    sym := zbar_symbol_next(sym);
  end;
End;

procedure TThreadZBar.Execute;
Var VideoDevice: String;
  iRC, iActive: Integer;
begin
  ZBarProc := zbar_processor_create(0);
  If Not Assigned(ZBarProc) Then
  Begin
    fLog := 'E: ZBar, unable to allocate memory.';
    Synchronize(@DoLogExt);
    Synchronize(@Termi);
    Exit;
  end;
  zbar_processor_set_data_handler(ZBarProc, @ZBarDataHandler, Nil);
  VideoDevice := '';
  if (zbar_processor_init(ZBarProc, PChar(VideoDevice), 1) <> 0) Or
    (zbar_processor_set_visible(ZBarProc, 1) <> 0) Then
  Begin
    iRC := zbar_processor_error_spew(ZBarProc, 0);
    If iRC = 1 Then
      fLog := 'E: Webcam nicht gefunden oder angeschlossen!'
    Else fLog := 'E: ZBar zbar_processor_init(): ' + IntToStr(iRC);
    Synchronize(@DoLogExt);
    Synchronize(@Termi);
    Exit;
  end;
  if zbar_processor_set_active(ZBarProc, 1) <> 0 Then
  Begin
    fLog := 'E: ZBar zbar_processor_set_active(): ' + IntToStr(zbar_processor_error_spew(ZBarProc, 0));
    Synchronize(@DoLogExt);
    Synchronize(@Termi);
    Exit;
  end;
  iActive := 1;
  Repeat
    iRC := zbar_processor_user_wait(ZBarProc, -1);
    if iRC >= 0 Then
    Begin
      if (iRC = Ord('q')) Or (iRC = Ord('Q')) Then
        break;
      if iRC = Ord(' ') Then
      Begin
        If iActive = 1 Then
          iActive := 0
        Else iActive := 1;
        if zbar_processor_set_active(ZBarProc, iActive) <> 0 Then
          break;
      end;
    end;
    Sleep(10);
  until (iRC < 0) Or Terminated;
  zbar_processor_set_data_handler(ZBarProc, Nil, Nil);
  if Not((iRC <> Ord('q')) And (iRC <> Ord('Q')) And
    (zbar_processor_get_error_code(ZBarProc) <> ZBAR_ERR_CLOSED)) Then
    zbar_processor_destroy(ZBarProc);
  ZBarProc := Nil;
  Synchronize(@Termi);
end;
// Thread für Barcode Reader
////////////////////////////////////////////////////////////////////

end.

